@@ -1,16 +1,16 @@
 use bollard::{
+    Docker,
     container::{
         ListContainersOptions, LogsOptions, MemoryStatsStats, RemoveContainerOptions,
         StartContainerOptions, Stats, StatsOptions,
     },
     service::ContainerSummary,
-    Docker,
 };
 use futures_util::StreamExt;
 use parking_lot::Mutex;
 use std::{
     collections::HashMap,
-    sync::{atomic::AtomicUsize, Arc},
+    sync::{Arc, atomic::AtomicUsize},
 };
 use tokio::{
     sync::mpsc::{Receiver, Sender},
@@ -19,11 +19,11 @@ use tokio::{
 use uuid::Uuid;
 
 use crate::{
+    ENTRY_POINT,
     app_data::{AppData, ContainerId, DockerCommand, State},
     app_error::AppError,
-    parse_args::CliArgs,
+    config::Config,
     ui::{GuiState, Status},
-    ENTRY_POINT,
 };
 mod message;
 pub use message::DockerMessage;
@@ -46,7 +46,7 @@ impl SpawnId {
 /// Cpu & Mem stats take twice as long as the update interval to get a value, so will have two being executed at the same time
 /// SpawnId::Stats takes container_id and binate value to enable both cycles of the same container_id to be inserted into the hashmap
 /// Binate value is toggled when all handles have been spawned off
-/// Also effectively means that the docker_update interval minimum will be 1000ms
+/// Also effectively means that the minimum docker_update interval will be 1000ms
 #[derive(Debug, Clone, Copy, Eq, Hash, PartialEq)]
 enum Binate {
     One,
@@ -64,8 +64,8 @@ impl Binate {
 
 pub struct DockerData {
     app_data: Arc<Mutex<AppData>>,
-    args: CliArgs,
     binate: Binate,
+    config: Config,
     docker: Arc<Docker>,
     gui_state: Arc<Mutex<GuiState>>,
     receiver: Receiver<DockerMessage>,
@@ -214,11 +214,11 @@ impl DockerData {
             .into_iter()
             .filter_map(|f| match f.id {
                 Some(_) => {
-                    if self.args.in_container
+                    if self.config.in_container
                         && f.command
                             .as_ref()
-                            .map_or(false, |c| c.starts_with(ENTRY_POINT))
-                        && self.args.show_self
+                            .is_some_and(|c| c.starts_with(ENTRY_POINT))
+                        && self.config.show_self
                     {
                         None
                     } else {
@@ -271,7 +271,7 @@ impl DockerData {
                 Arc::clone(&self.app_data);
             let docker = Arc::clone(&self.docker);
             let spawns = Arc::clone(&self.spawns);
-            let std_err = self.args.std_err;
+            let std_err = self.config.show_std_err;
             let init = Arc::clone(&init);
             self.spawns.lock().insert(
                 SpawnId::Log(id.clone()),
@@ -319,7 +319,7 @@ impl DockerData {
                     container.id.clone(),
                     last_updated,
                     Arc::clone(&self.spawns),
-                    self.args.std_err,
+                    self.config.show_std_err,
                 )));
             }
         };
@@ -350,6 +350,7 @@ impl DockerData {
             GuiState::start_loading_animation(&gui_state, uuid);
             if match control {
                 DockerCommand::Delete => {
+                    gui_state.lock().set_delete_container(None);
                     docker
                         .remove_container(
                             id.get(),
@@ -399,8 +400,9 @@ impl DockerData {
     }
 
     /// Send an update message every x ms, where x is the args.docker_interval
-    fn heartbeat(args: &CliArgs, docker_tx: Sender<DockerMessage>) {
-        let update_duration = std::time::Duration::from_millis(u64::from(args.docker_interval));
+    fn heartbeat(config: &Config, docker_tx: Sender<DockerMessage>) {
+        let update_duration =
+            std::time::Duration::from_millis(u64::from(config.docker_interval_ms));
         let mut now = std::time::Instant::now();
         tokio::spawn(async move {
             loop {
@@ -421,11 +423,11 @@ impl DockerData {
         docker_tx: Sender<DockerMessage>,
         gui_state: Arc<Mutex<GuiState>>,
     ) {
-        let args = app_data.lock().args.clone();
+        let args = app_data.lock().config.clone();
         if app_data.lock().get_error().is_none() {
             let mut inner = Self {
                 app_data,
-                args,
+                config: args,
                 binate: Binate::One,
                 docker: Arc::new(docker),
                 gui_state,
@@ -433,7 +435,7 @@ impl DockerData {
                 spawns: Arc::new(Mutex::new(HashMap::new())),
             };
             inner.initialise_container_data().await;
-            Self::heartbeat(&inner.args, docker_tx);
+            Self::heartbeat(&inner.config, docker_tx);
             inner.message_handler().await;
         }
     }
