@@ -2,18 +2,18 @@ use anyhow::Result;
 use crossterm::{
     event::{self, DisableMouseCapture, Event},
     execute,
-    terminal::{disable_raw_mode, enable_raw_mode, EnterAlternateScreen, LeaveAlternateScreen},
+    terminal::{EnterAlternateScreen, LeaveAlternateScreen, disable_raw_mode, enable_raw_mode},
 };
 use parking_lot::Mutex;
 use ratatui::{
+    Frame, Terminal,
     backend::CrosstermBackend,
     layout::{Constraint, Direction, Layout, Position},
-    Frame, Terminal,
 };
 use std::{
     collections::HashSet,
     io::{self, Stdout, Write},
-    sync::{atomic::Ordering, Arc},
+    sync::{Arc, atomic::Ordering},
     time::Duration,
 };
 use std::{sync::atomic::AtomicBool, time::Instant};
@@ -23,6 +23,8 @@ use tracing::error;
 mod color_match;
 mod draw_blocks;
 mod gui_state;
+mod redraw;
+pub use redraw::Redraw;
 
 pub use self::color_match::*;
 pub use self::gui_state::{DeleteButton, GuiState, SelectablePanel, Status};
@@ -32,21 +34,24 @@ use crate::{
         SortedOrder, State,
     },
     app_error::AppError,
+    config::{AppColors, Keymap},
     exec::TerminalSize,
     input_handler::InputMessages,
 };
 
-pub const ORANGE: ratatui::style::Color = ratatui::style::Color::Rgb(255, 178, 36);
-const POLL_RATE: Duration = std::time::Duration::from_millis(100);
+const POLL_RATE: Duration = std::time::Duration::from_millis(50);
+
+// could have a render struct, which takes in poll rate, and docker
 
 pub struct Ui {
     app_data: Arc<Mutex<AppData>>,
+    cursor_position: Position,
     gui_state: Arc<Mutex<GuiState>>,
     input_tx: Sender<InputMessages>,
     is_running: Arc<AtomicBool>,
     now: Instant,
+    redraw: Arc<Redraw>,
     terminal: Terminal<CrosstermBackend<Stdout>>,
-    cursor_position: Position,
 }
 
 impl Ui {
@@ -68,26 +73,31 @@ impl Ui {
         gui_state: Arc<Mutex<GuiState>>,
         input_tx: Sender<InputMessages>,
         is_running: Arc<AtomicBool>,
+        redraw: Arc<Redraw>,
     ) {
-        if let Ok(mut terminal) = Self::setup_terminal() {
-            let cursor_position = terminal.get_cursor_position().unwrap_or_default();
-            let mut ui = Self {
-                app_data,
-                cursor_position,
-                gui_state,
-                input_tx,
-                is_running,
-                now: Instant::now(),
-                terminal,
-            };
-            if let Err(e) = ui.draw_ui().await {
-                error!("{e}");
+        match Self::setup_terminal() {
+            Ok(mut terminal) => {
+                let cursor_position = terminal.get_cursor_position().unwrap_or_default();
+                let mut ui = Self {
+                    app_data,
+                    cursor_position,
+                    gui_state,
+                    input_tx,
+                    is_running,
+                    now: Instant::now(),
+                    redraw,
+                    terminal,
+                };
+                if let Err(e) = ui.draw_ui().await {
+                    error!("{e}");
+                }
+                if let Err(e) = ui.reset_terminal() {
+                    error!("{e}");
+                };
+            }
+            _ => {
+                error!("Terminal Error");
             }
-            if let Err(e) = ui.reset_terminal() {
-                error!("{e}");
-            };
-        } else {
-            error!("Terminal Error");
         }
     }
 
@@ -124,22 +134,37 @@ impl Ui {
     /// Draw the the error message ui, for 5 seconds, with a countdown
     fn err_loop(&mut self) -> Result<(), AppError> {
         let mut seconds = 5;
+        let colors = self.app_data.lock().config.app_colors;
+        let keymap = self.app_data.lock().config.keymap.clone();
+        let mut redraw = true;
         loop {
             if self.now.elapsed() >= std::time::Duration::from_secs(1) {
                 seconds -= 1;
                 self.now = Instant::now();
+                redraw = true;
                 if seconds < 1 {
                     break;
                 }
             }
 
-            if self
-                .terminal
-                .draw(|f| draw_blocks::error(f, AppError::DockerConnect, Some(seconds)))
-                .is_err()
+            if redraw
+                && self
+                    .terminal
+                    .draw(|f| {
+                        draw_blocks::error::draw(
+                            colors,
+                            &AppError::DockerConnect,
+                            f,
+                            &keymap,
+                            Some(seconds),
+                        );
+                    })
+                    .is_err()
             {
                 return Err(AppError::Terminal);
             }
+            redraw = false;
+            std::thread::sleep(POLL_RATE);
         }
         Ok(())
     }
@@ -163,21 +188,41 @@ impl Ui {
         self.gui_state.lock().status_del(Status::Exec);
     }
 
+    /// Use the previously redrawn time, the current time, the docker_interval, and the redraw struct, to calculate
+    /// if the screen should be redrawn or not
+    fn should_redraw(&self, previous: &mut Instant, docker_interval_ms: u128) -> bool {
+        let result = self.redraw.swap() || previous.elapsed().as_millis() >= docker_interval_ms;
+        if result {
+            *previous = std::time::Instant::now();
+        }
+        result
+    }
+
     /// The loop for drawing the main UI to the terminal
     async fn gui_loop(&mut self) -> Result<(), AppError> {
-        while self.is_running.load(Ordering::SeqCst) {
-            let fd = FrameData::from(&*self);
-            let exec = fd.status.contains(&Status::Exec);
-            if exec {
-                self.exec().await;
-            }
+        let colors = self.app_data.lock().config.app_colors;
+        let keymap = self.app_data.lock().config.keymap.clone();
+        let docker_interval_ms = u128::from(self.app_data.lock().config.docker_interval_ms);
+        let mut drawn_at = std::time::Instant::now();
 
-            if self
-                .terminal
-                .draw(|frame| draw_frame(frame, &self.app_data, &self.gui_state, &fd))
-                .is_err()
-            {
-                return Err(AppError::Terminal);
+        while self.is_running.load(Ordering::SeqCst) {
+            if self.should_redraw(&mut drawn_at, docker_interval_ms) {
+                let fd = FrameData::from(&*self);
+
+                let exec = fd.status.contains(&Status::Exec);
+                if exec {
+                    self.exec().await;
+                }
+
+                if self
+                    .terminal
+                    .draw(|frame| {
+                        draw_frame(&self.app_data, colors, &keymap, frame, &fd, &self.gui_state);
+                    })
+                    .is_err()
+                {
+                    return Err(AppError::Terminal);
+                }
             }
 
             if crossterm::event::poll(POLL_RATE).unwrap_or(false) {
@@ -224,6 +269,7 @@ impl Ui {
 #[derive(Debug, Clone)]
 pub struct FrameData {
     chart_data: Option<(CpuTuple, MemTuple)>,
+    color_logs: bool,
     columns: Columns,
     container_title: String,
     delete_confirm: Option<ContainerId>,
@@ -258,6 +304,7 @@ impl From<&Ui> for FrameData {
         let (filter_by, filter_term) = app_data.get_filter();
         Self {
             chart_data: app_data.get_chart_data(),
+            color_logs: app_data.config.color_logs,
             columns: app_data.get_width(),
             container_title: app_data.get_container_title(),
             delete_confirm: gui_data.get_delete_container(),
@@ -281,10 +328,12 @@ impl From<&Ui> for FrameData {
 
 /// Draw the main ui to a frame of the terminal
 fn draw_frame(
-    f: &mut Frame,
     app_data: &Arc<Mutex<AppData>>,
-    gui_state: &Arc<Mutex<GuiState>>,
+    colors: AppColors,
+    keymap: &Keymap,
+    f: &mut Frame,
     fd: &FrameData,
+    gui_state: &Arc<Mutex<GuiState>>,
 ) {
     let whole_constraints = if fd.status.contains(&Status::Filter) {
         vec![Constraint::Max(1), Constraint::Min(1), Constraint::Max(1)]
@@ -326,15 +375,15 @@ fn draw_frame(
         .constraints(lower_split)
         .split(upper_main[1]);
 
-    draw_blocks::containers(app_data, top_panel[0], f, fd, gui_state);
+    draw_blocks::containers::draw(app_data, top_panel[0], colors, f, fd, gui_state);
 
-    draw_blocks::logs(app_data, lower_main[0], f, fd, gui_state);
+    draw_blocks::logs::draw(app_data, lower_main[0], colors, f, fd, gui_state);
 
-    draw_blocks::heading_bar(whole_layout[0], f, fd, gui_state);
+    draw_blocks::headers::draw(whole_layout[0], colors, f, fd, gui_state, keymap);
 
     // Draw filter bar
     if let Some(rect) = whole_layout.get(2) {
-        draw_blocks::filter_bar(*rect, f, fd);
+        draw_blocks::filter::draw(*rect, colors, f, fd);
     }
 
     if let Some(id) = fd.delete_confirm.as_ref() {
@@ -345,14 +394,14 @@ fn draw_frame(
                 gui_state.lock().set_delete_container(None);
             },
             |name| {
-                draw_blocks::delete_confirm(f, gui_state, name);
+                draw_blocks::delete_confirm::draw(colors, f, gui_state, keymap, name);
             },
         );
     }
 
     // only draw commands + charts if there are containers
     if let Some(rect) = top_panel.get(1) {
-        draw_blocks::commands(app_data, *rect, f, fd, gui_state);
+        draw_blocks::commands::draw(app_data, *rect, colors, f, fd, gui_state);
 
         // Can calculate the max string length here, and then use that to keep the ports section as small as possible (+4 for some padding + border)
         let ports_len =
@@ -364,20 +413,27 @@ fn draw_frame(
             .constraints([Constraint::Min(1), Constraint::Max(ports_len)])
             .split(lower_main[1]);
 
-        draw_blocks::chart(f, lower[0], fd);
-        draw_blocks::ports(f, lower[1], fd);
+        draw_blocks::charts::draw(lower[0], colors, f, fd);
+        draw_blocks::ports::draw(lower[1], colors, f, fd);
     }
 
     if let Some((text, instant)) = fd.info_text.as_ref() {
-        draw_blocks::info(f, text.to_owned(), instant, gui_state);
+        draw_blocks::info::draw(colors, f, gui_state, instant, text.to_owned());
     }
 
     // Check if error, and show popup if so
     if fd.status.contains(&Status::Help) {
-        draw_blocks::help_box(f);
+        let tz = app_data.lock().config.timezone.clone();
+        draw_blocks::help::draw(
+            colors,
+            f,
+            keymap,
+            app_data.lock().config.show_timestamp,
+            tz.as_ref(),
+        );
     }
 
-    if let Some(error) = fd.has_error {
-        draw_blocks::error(f, error, None);
+    if let Some(error) = fd.has_error.as_ref() {
+        draw_blocks::error::draw(colors, error, f, keymap, None);
     }
 }
